#!/usr/bin/env python3
# script to produce figures for LVK O3 Sco X-1 CrossCorr paper

# Copyright (C) 2022 John T. Whelan

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with with program; see the file COPYING. If not, see
# <https://www.gnu.org/licenses/>.

from argparse import ArgumentParser
from configparser import ConfigParser
from lal import gpstime
import numpy as np
import matplotlib
matplotlib.use("Agg")
from matplotlib import rc
matplotlib.rc('text', usetex=True)
matplotlib.rc('font', family='serif')
import matplotlib.pyplot as plt
import matplotlib.font_manager as ftmgr
from numpy.testing import assert_equal
from scipy import stats

# scinotstring = latexscinot(format,number)
# Converts a number into a LaTeX scientific notation string
def latexscinot(format,number):
    if (number == 0):
        scinotstring = '0';
    else:
        exponent = np.floor(np.log10(np.abs(number)));
        if (exponent == 0):
            scinotstring = format % number;
        else:
            scinotstring = ( (format + r'\times 10^{%d}')
                             % (number/10**exponent, exponent) );
    return scinotstring

## This conversion uses the convention that h0eff^2 <= h0^2
## (i.e., they are equal for circular polarization)
def h0effsq_by_h0sq(cosisq):
    return 0.5 * ( ( 0.5*(1.+cosisq) )**2 + cosisq )

def parseMetadata(metadataFile,columnSelection_s):
    with open(metadataFile) as f:
        header = f.readline().lstrip('#')
    columnName_c = header.split()
    columnNumber_n = dict([(n,c) for c,n in enumerate(columnName_c)])
    columnNumber_s = [columnNumber_n[n] for n in columnSelection_s]
    data_s = dict()
    numSel = len(columnSelection_s)
    for s in range(numSel):
        data_s[columnSelection_s[s]] = np.loadtxt(metadataFile,usecols=columnNumber_s[s],unpack=True)
        if (data_s[columnSelection_s[s]].shape == ()):
            data_s[columnSelection_s[s]] = np.array([data_s[columnSelection_s[s]]])
    return data_s

# Sky position of Sco X-1
ScoX1raHMS = [16.,19.,55.0850]
ScoX1negdecDMS = [15.,38.,24.9]

ScoX1raHr = ScoX1raHMS[0] + (ScoX1raHMS[1] + ScoX1raHMS[2]/60.)/60.
ScoX1decDeg = - (ScoX1negdecDMS[0]
                  + (ScoX1negdecDMS[1] + ScoX1negdecDMS[2]/60.)/60.)

# Start and end times of O3a and O3b
# See https://wiki.ligo.org/LSC/JRPComm/ObsRun3#Schedule
# and https://www.gw-openscience.org/O3/
O3aStartGPS, O3aEndGPS = np.loadtxt('O3aTimesActual.dat')
O3bStartGPS, O3bEndGPS = np.loadtxt('O3bTimesActual.dat')

# Tmax values from O1
(
    fLo_C,fHi_C,K1Lo_C,K1Hi_C,TLoSig_C,THiSig_C,Tsft_C,Tmax_C
) = np.loadtxt('O1TmaxSummary.dat',unpack=True)

# Frequency boundaries for different freqParams ini files
boundaries_F = np.array([25,50,100,150,200,300,400,
                         600,800,1000,1200,1400,1600])
fLo_F = boundaries_F[:-1]
fHi_F = boundaries_F[1:]
Nbands = len(fLo_F)

cp_F = []
Tmax_F_c = []
TascLo_F_c = []
TascHi_F_c = []
apLo_F_c = []
apHi_F_c = []
Tsft_F = np.empty(Nbands)
TmaxLo_F = np.empty(Nbands)
TmaxHi_F = np.empty(Nbands)
nCells_F = np.empty(Nbands,dtype=int)
fLoSFTs_F,fHiSFTs_F,TsftSFTs_F,nSFTsH1_F,nSFTsL1_F = np.loadtxt(
    "O3NumSFTs.dat", unpack=True
)

for i in range(Nbands):
    cp_F.append(ConfigParser())
    cp_F[i].optionxform = str
    cp_F[i].read('freqParams_%04d_%04dHz.ini' % (fLo_F[i],fHi_F[i]))
    if i == 0:
        WangPorbSec = cp_F[i].getfloat('parameters','orbitPSecCenter')
        WangdPorbSec = cp_F[i].getfloat('parameters','orbitPSecSigma')
        WangTascGPS = cp_F[i].getfloat('parameters','orbitTimeAscCenter')
        WangdTascSec = cp_F[i].getfloat('parameters','orbitTimeAscSigma')
        norb = cp_F[i].getint('parameters','norb')
        mSec = 1. / 299792458. # one meter, in seconds
        kmSec = 1.e3 * mSec
        WangK1perap = 2.*np.pi/WangPorbSec / kmSec
        TascGPS = norb * WangPorbSec + WangTascGPS
        dTascSec = np.sqrt(WangdTascSec ** 2 + (norb * WangdPorbSec) ** 2)
        daySec = 24 * 60 * 60
    else:
        assert_equal(WangPorbSec,
                     cp_F[i].getfloat('parameters','orbitPSecCenter'))
        assert_equal(WangdPorbSec,
                     cp_F[i].getfloat('parameters','orbitPSecSigma'))
        assert_equal(WangTascGPS,
                     cp_F[i].getfloat('parameters','orbitTimeAscCenter'))
        assert_equal(WangdTascSec,
                     cp_F[i].getfloat('parameters','orbitTimeAscSigma'))
    K1Lo_c = np.array([float(a) for a in
                       cp_F[i].get('parameters','K1Lo').split(',')])
    K1Hi_c = np.array([float(a) for a in
                       cp_F[i].get('parameters','K1Hi').split(',')])
    TLoSig_c = np.array([float(a) for a in
                         cp_F[i].get('parameters','TLoSig').split(',')])
    THiSig_c = np.array([float(a) for a in
                         cp_F[i].get('parameters','THiSig').split(',')])
    nCells = len(K1Lo_c)
    nCells_F[i] = nCells
    Tmax_F_c.append(np.empty(nCells))
    for ic in range(nCells):
        cellMask_C = (
            (fLo_F[i]>=fLo_C) & (fHi_F[i]<=fHi_C)
            & (K1Lo_c[ic]>=K1Lo_C) & (K1Hi_c[ic]<=K1Hi_C)
            & (TLoSig_c[ic]>=TLoSig_C) & (THiSig_c[ic]<=THiSig_C)
        )
        if (sum(cellMask_C) != 1):
            raise ValueError(('%d rows in Tmax file match; should be 1'
                              % sum(cellMask_C)))
        Tmax_F_c[i][ic] = Tmax_C[cellMask_C]
        # The Tsft values should all be the same for a band
        Tsft_F[i] = Tsft_C[cellMask_C]
    if ( (fLo_F[i] >= 400) & (fHi_F[i] <= 600) ):
        Tmax_F_c[i] *=2
    assert_equal(fLo_F[i],fLoSFTs_F[i])
    assert_equal(fHi_F[i],fHiSFTs_F[i])
    assert_equal(Tsft_F[i],TsftSFTs_F[i])
    print( '%d to %d Hz: %d+%d=%d total SFTs;'
           % (fLo_F[i],fHi_F[i],
              nSFTsH1_F[i],nSFTsL1_F[i],nSFTsH1_F[i]+nSFTsL1_F[i]) )
    TobsH1 = nSFTsH1_F[i] * Tsft_F[i]
    print( '  H1: %d*%ds = %d seconds = %.1f days'
           % (nSFTsH1_F[i],Tsft_F[i],TobsH1,TobsH1/daySec) )
    TobsL1 = nSFTsL1_F[i] * Tsft_F[i]
    print( '  L1: %d*%ds = %d seconds = %.1f days'
           % (nSFTsL1_F[i],Tsft_F[i],TobsL1,TobsL1/daySec) )

    TmaxLo_F[i] = min(Tmax_F_c[i])
    TmaxHi_F[i] = max(Tmax_F_c[i])

    TascLo_c = TascGPS + dTascSec * TLoSig_c
    TascHi_c = TascGPS + dTascSec * THiSig_c
    apLo_c = K1Lo_c / WangK1perap
    apHi_c = K1Hi_c / WangK1perap

    TascLo_F_c.append(TascLo_c)
    TascHi_F_c.append(TascHi_c)
    apLo_F_c.append(apLo_c)
    apHi_F_c.append(apHi_c)

TmaxMin = np.min(TmaxLo_F)
TmaxMax = np.max(TmaxHi_F)

# Fractional loss of SNR due to spin wandering
# from Whelan et al PRD 91, 102005 (2015), eqn (5.9)
Trun = O3bEndGPS-O3aStartGPS
SWLoss = (np.pi**2/6.) * Trun * 1e6 * (1e-12)**2 * (TmaxMax)**2

TPEllipseRadius = 3.3

daytimefmt = '%Y--%b--%d %T\,UTC'
dayfmt = '%Y--%b--%d'

Porigin = WangPorbSec
Torigin = TascGPS
dPtildeSec = WangdPorbSec * WangdTascSec / dTascSec
dPorbdTascShearCorr = norb * (WangdPorbSec/dTascSec)**2
dPorbdTascShearUsed = norb / (norb**2 + (WangdPorbSec/WangdTascSec)**2)

apMin = min(apLo_F_c[0])
apMax = max(apHi_F_c[0])

# Torque balance numbers
BildstenHc600Hz = 4e-27 * np.sqrt(39)
BradyH0600Hz = np.sqrt(15./32.) * BildstenHc600Hz
h0600Hz = 2. * BradyH0600Hz

(
    fmid_u, nTmplt_u
) = np.loadtxt('O3NumTemplates.dat',usecols=(0,1),unpack=True)

(O1fmin_D,O1fmax_D,O1nTmplt_D) = np.loadtxt('O1templatecount.txt',unpack=True)
O1thresh50Hz_D = stats.norm.isf(0.1/O1nTmplt_D)
numDags = len(O1fmin_D)
nTmplt_D = np.array([
    np.sum(nTmplt_u[(O1fmin_D[D] <= fmid_u) & (fmid_u <= O1fmax_D[D])])
    for D in range(numDags)
])
thresh50Hz_D = stats.norm.isf(0.1/nTmplt_D)

fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)
for D in range(numDags):
    if D==0:
        ax.semilogx([O1fmin_D[D],O1fmax_D[D]],
                    [O1thresh50Hz_D[D],O1thresh50Hz_D[D]],'y-',color='C1',
                    label='O1');
    else:
        ax.semilogx([O1fmin_D[D],O1fmax_D[D]],
                    [O1thresh50Hz_D[D],O1thresh50Hz_D[D]],'y-',color='C1');
    if nTmplt_D[D] > 0:
        if D==0:
            ax.semilogx([O1fmin_D[D],O1fmax_D[D]],
                        [thresh50Hz_D[D],thresh50Hz_D[D]],'b-',color='C0',
                        label='O3');
        else:
            ax.semilogx([O1fmin_D[D],O1fmax_D[D]],
                        [thresh50Hz_D[D],thresh50Hz_D[D]],'b-',color='C0');
ax.set_xlabel('GW Frequency (Hz)');
ax.set_ylabel(r'$\rho$ threshold');
ax.set_title(r'One expected Gaussian false alarm per $50$Hz band');
ax.semilogx([25,400,400,600,600,2000],
            [6.5,6.5,6.2,6.2,6.0,6.0],'m--',label='O1 followup')
ax.semilogx([25,400,400,600,600,1600],
            [6.3,6.3,6.2,6.2,5.8,5.8],'k--',label='O3 followup')
tickvals = [25,50,100,200,500,1000,2000]
ticklabs = ['%d' % x for x in tickvals]
ax.set_xticks(tickvals)
ax.set_xticklabels(ticklabs)
# ax.set_ylim([5.5,7])
ax.set_xlim([25,2000])
ax.legend(loc='lower left')
ax.grid(True);
fig.savefig('threshold.eps',bbox_inches='tight')

fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)
for D in range(numDags):
    if D==0:
        ax.loglog([O1fmin_D[D],O1fmax_D[D]],
                    [O1nTmplt_D[D],O1nTmplt_D[D]],'y-',color='C1',
                    label='O1');
    else:
        ax.loglog([O1fmin_D[D],O1fmax_D[D]],
                    [O1nTmplt_D[D],O1nTmplt_D[D]],'y-',color='C1');
    if nTmplt_D[D] > 0:
        if D==0:
            ax.loglog([O1fmin_D[D],O1fmax_D[D]],
                        [nTmplt_D[D],nTmplt_D[D]],'b-',color='C0',
                        label='O3');
        else:
            ax.loglog([O1fmin_D[D],O1fmax_D[D]],
                        [nTmplt_D[D],nTmplt_D[D]],'b-',color='C0');
ax.set_xlabel('GW Frequency (Hz)');
ax.set_ylabel(r'Number of Templates in $5$Hz Band');
tickvals = [25,50,100,200,500,1000,2000]
ticklabs = ['%d' % x for x in tickvals]
ax.set_xticks(tickvals)
ax.set_xticklabels(ticklabs)
# ax.set_ylim([5.5,7])
ax.set_xlim([25,2000])
ax.legend(loc='lower left')
ax.grid(True);
fig.savefig('numTemplates.eps',bbox_inches='tight')

for i in range(Nbands):
    fig = plt.figure();
    fig.set_size_inches(5,3);

    ax = fig.add_subplot(111)

    for ic in range(nCells_F[i]):
        amn = apLo_F_c[i][ic]
        amx = apHi_F_c[i][ic]
        Tmn = TascLo_F_c[i][ic]
        Tmx = TascHi_F_c[i][ic]
        cellColor = 0.7 * (
            1.-(np.ones(3)*(Tmax_F_c[i][ic]-TmaxMin)
                / (TmaxMax-TmaxMin) )
        )
#         print cellColor
        ax.fill(np.array([Tmn,Tmn,Tmx,Tmx,Tmn])-Torigin,
                np.array([amn,amx,amx,amn,amn]),
                color=cellColor,
                lw=1,ls='solid',ec='k')
        if cellColor[0] < 0.5:
            textColor = 'w'
        else:
            textColor = 'k'
        ax.text(0.5*(Tmn+Tmx)-Torigin,0.5*(amn+amx),
                ('%d' % Tmax_F_c[i][ic]),
                ha='center',va='center',color = textColor)

    ax.set_ylabel(r'$a\sin i$ (lt-s)')
    ax.set_xlabel(r"$t'_{\rm asc}-%.0f$ (s)" % Torigin)
    ax.set_title(r'$T_{\rm max}$ (s) for %d-%d\,Hz'
                 % (fLo_F[i],fHi_F[i]))

    ax.set_axisbelow(True)
    ax.grid(True)

    fig.savefig(('maxLags_%04d_%04d.eps' % (fLo_F[i],fHi_F[i],)),
                bbox_inches='tight')
    plt.close(fig)


k_k = np.arange(1,4)
col_k = ['b','g','r']
# col_k = ['C0']
nK = len(k_k)
if len(col_k) < nK:
    col_k = np.tile(col_k,int(np.ceil(nK/len(col_k))))
if len(col_k) > nK:
    col_k = np.resize(col_k,nK)

phi_p = np.linspace(0,2*np.pi,100)
cosphi_p = np.cos(phi_p)
sinphi_p = np.sin(phi_p)

WangTsig_kp = WangTascGPS + k_k[:,None] * WangdTascSec * cosphi_p[None,:]
Psig_kp = WangPorbSec + k_k[:,None] * WangdPorbSec * sinphi_p[None,:]
Tsig_kp = WangTsig_kp + norb * Psig_kp
# Ptildesig_kp = Psig_kp - norb*WangdPorbSec**2*(Tsig_kp-TascGPS)/dTascSec**2
Ptildesig_kp = Psig_kp - dPorbdTascShearUsed*(Tsig_kp-TascGPS)

O1cp = ConfigParser()
O1cp.optionxform = str
O1cp.read('O1_freqParams_0025_0050Hz_v2.ini')

O1NCells = 18

O1TascLo_c = np.array( [ O1cp.getfloat(('lagTime%d' % c),'TascMin')
                          for c in range(O1NCells) ] )

O1TascHi_c = np.array( [ O1cp.getfloat(('lagTime%d' % c),'TascMax')
                          for c in range(O1NCells) ] )

O1Torigin = 0.5*(min(O1TascLo_c)+max(O1TascHi_c))

O1O3norb = int(round((Torigin - O1Torigin)/WangPorbSec))

O1PorbLo_c = np.array( [ O1cp.getfloat(('lagTime%d' % c),'pMin')
                          for c in range(O1NCells) ] )

O1PorbHi_c = np.array( [ O1cp.getfloat(('lagTime%d' % c),'pMax')
                          for c in range(O1NCells) ] )

# Parameters for Zhang et al O2 search, from paper

# O2PorbLo = 68023.86 - 3 * 0.04
# O2PorbHi = 68023.86 + 3 * 0.04

# Assuming Porb search region used the more precise values from Wang et al
O2PorbLo = WangPorbSec - 3 * WangdPorbSec
O2PorbHi = WangPorbSec + 3 * WangdPorbSec

O2TascLo = 1178556229 - 3 * 139
O2TascHi = 1178556229 + 3 * 139

O2Torigin = 1178556229

O2O3norb = int(round((Torigin - O2Torigin)/WangPorbSec))

fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)

for k in range(nK):
    ax.plot(Tsig_kp[k]-Torigin,Psig_kp[k]-Porigin,color=col_k[k],lw=1)

O1TascMin = min(O1TascLo_c)
O1TascMax = max(O1TascHi_c)
O1PorbMin = min(O1PorbLo_c)
O1PorbMax = max(O1PorbHi_c)

# for c in range(O1NCells):
#     ax.plot(np.array([O1TascLo_c[c]+O1O3norb*O1PorbLo_c[c],
#                       O1TascLo_c[c]+O1O3norb*O1PorbHi_c[c],
#                       O1TascHi_c[c]+O1O3norb*O1PorbHi_c[c],
#                       O1TascHi_c[c]+O1O3norb*O1PorbLo_c[c],
#                       O1TascLo_c[c]+O1O3norb*O1PorbLo_c[c]])-Torigin,
#             np.array([O1PorbLo_c[c],O1PorbHi_c[c],O1PorbHi_c[c],
#                       O1PorbLo_c[c],O1PorbLo_c[c]])-Porigin,'k--',
#             color=[0.5,0.5,0.5])

# Just plot limits of O1 search, not full regions
ax.plot(np.array([O1TascMin+O1O3norb*O1PorbMin,
                  O1TascMin+O1O3norb*O1PorbMax,
                  O1TascMax+O1O3norb*O1PorbMax,
                  O1TascMax+O1O3norb*O1PorbMin,
                  O1TascMin+O1O3norb*O1PorbMin])-Torigin,
        np.array([O1PorbMin,O1PorbMax,O1PorbMax,
                  O1PorbMin,O1PorbMin])-Porigin,'k--',
        color=[0.5,0.5,0.5])

ax.plot(np.array([O2TascLo+O2O3norb*O2PorbLo,
                  O2TascLo+O2O3norb*O2PorbHi,
                  O2TascHi+O2O3norb*O2PorbHi,
                  O2TascHi+O2O3norb*O2PorbLo,
                  O2TascLo+O2O3norb*O2PorbLo])-Torigin,
        np.array([O2PorbLo,O2PorbHi,O2PorbHi,
                  O2PorbLo,O2PorbLo])-Porigin,':',
        color='C1')

for c in [0,2,7]:
    Tmn = TascLo_F_c[-1][c]
    Tmx = TascHi_F_c[-1][c]
    Tplot_t = np.linspace(Tmn,Tmx,100)
#     Pmid_t = WangPorbSec + norb*WangdPorbSec**2*(Tplot_t-TascGPS)/dTascSec**2
    Pmid_t = WangPorbSec + dPorbdTascShearUsed*(Tplot_t-TascGPS)
    Pwid_t = WangdPorbSec * WangdTascSec/dTascSec * np.sqrt(
        TPEllipseRadius**2 - ((Tplot_t-TascGPS)/dTascSec)**2
    )
    Pmn_t = Pmid_t - Pwid_t
    Pmx_t = Pmid_t + Pwid_t
    ax.plot(np.concatenate(([Tplot_t[0]],Tplot_t,
                            [Tplot_t[-1]],Tplot_t[::-1]))-Torigin,
            np.concatenate(([Pmn_t[0]],Pmx_t,
                            [Pmn_t[-1]],Pmn_t[::-1]))-Porigin,
            'k-')

ax.set_title('Search Regions for O1, O2 and O3')
ax.set_xlabel(r"$t'_{\rm asc}-%.0f$ (s)" % Torigin)
ax.set_ylabel(r'$P_{\rm orb}-%.2f$ (s)' % Porigin)
ax.grid(True)

fig.savefig('TPRegionO1O3.eps',bbox_inches='tight')
plt.close()

fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)

for k in range(nK):
    ax.plot(Tsig_kp[k]-Torigin,Ptildesig_kp[k]-Porigin,color=col_k[k],lw=1)
    ax.text(Tsig_kp[k,90]-Torigin,Ptildesig_kp[k,90]-Porigin,
            ('$%d\sigma$' % k_k[k]),
            color=col_k[k])

for c in [0,2,7]:
    Tmn = TascLo_F_c[-1][c]
    Tmx = TascHi_F_c[-1][c]
    Tplot_t = np.linspace(Tmn,Tmx,100)
    Pmid_t = (
        WangPorbSec
    )
    Pwid_t = WangdPorbSec * WangdTascSec/dTascSec * np.sqrt(
        TPEllipseRadius**2 - ((Tplot_t-TascGPS)/dTascSec)**2
    )
    Pmn_t = Pmid_t - Pwid_t
    Pmx_t = Pmid_t + Pwid_t
    ax.plot(np.concatenate(([Tplot_t[0]],Tplot_t,
                            [Tplot_t[-1]],Tplot_t[::-1]))-Torigin,
            np.concatenate(([Pmn_t[0]],Pmx_t,
                            [Pmn_t[-1]],Pmn_t[::-1]))-Porigin,
            'k-')
    Pmn = WangPorbSec - TPEllipseRadius * WangdPorbSec * WangdTascSec/dTascSec
    Pmx = WangPorbSec + TPEllipseRadius * WangdPorbSec * WangdTascSec/dTascSec
    ax.plot(np.array([Tmn,Tmn,Tmx,Tmx,Tmn])-Torigin,
            np.array([Pmn,Pmx,Pmx,Pmn,Pmn])-Porigin,'k:')

ax.set_title('Search Region in Sheared Coordinates')
ax.set_xlabel(r"$t'_{\rm asc}-%.0f$ (s)" % Torigin)
ax.set_ylabel(r'$\tilde{P}-%.2f$ (s)' % Porigin)
ax.grid(True)

fig.savefig('TPRegionSheared.eps',bbox_inches='tight')
fig.savefig('TPRegionSheared.svg',bbox_inches='tight')
plt.close()

followupdata_s = parseMetadata(
    'search_followup_results_0025_1600_postlevel3.dat',
    ['freq','rho0','rho1','rho1_H1H1','rho1_L1L1',
     'rho2','rho2_H1H1','rho2_L1L1',
     'rho3','rho3_H1H1','rho3_L1L1','vetoed']
)

freq_c = followupdata_s['freq']
rho0_c = followupdata_s['rho0']
rho1_c = followupdata_s['rho1']
rho1H_c = followupdata_s['rho1_H1H1']
rho1L_c = followupdata_s['rho1_L1L1']
rho2_c = followupdata_s['rho2']
rho2H_c = followupdata_s['rho2_H1H1']
rho2L_c = followupdata_s['rho2_L1L1']
rho3_c = followupdata_s['rho3']
rho3H_c = followupdata_s['rho3_H1H1']
rho3L_c = followupdata_s['rho3_L1L1']
vetoNum_c = followupdata_s['vetoed']

injfollowupdata_s = parseMetadata(
    'inj_followup_results_0025_0500_postlevel3.dat',
    ['freq','rho0','rho1','rho1_H1H1','rho1_L1L1',
     'rho2','rho2_H1H1','rho2_L1L1',
     'rho3','rho3_H1H1','rho3_L1L1','vetoed']
)

injfreq_c = injfollowupdata_s['freq']
injrho0_c = injfollowupdata_s['rho0']
injrho1_c = injfollowupdata_s['rho1']
injrho1H_c = injfollowupdata_s['rho1_H1H1']
injrho1L_c = injfollowupdata_s['rho1_L1L1']
injrho2_c = injfollowupdata_s['rho2']
injrho2H_c = injfollowupdata_s['rho2_H1H1']
injrho2L_c = injfollowupdata_s['rho2_L1L1']
injrho3_c = injfollowupdata_s['rho3']
injrho3H_c = injfollowupdata_s['rho3_H1H1']
injrho3L_c = injfollowupdata_s['rho3_L1L1']
injvetoNum_c = injfollowupdata_s['vetoed']

xmin = 1.0
xmax = 3.1
ymin = 0.5
ymax = 2.6
x_x = np.linspace(xmin,xmax,100)

fig = plt.figure()
fig.set_size_inches(5,3)

ax = fig.add_subplot(111)

ax.plot(x_x,np.ones_like(x_x), 'b-')
ax.plot(x_x,4./x_x, 'g--')
ax.plot(x_x,2./x_x, 'g--')
ax.plot(rho2_c/rho1_c,rho3_c/rho2_c,'k.', label = 'search results')
ax.plot(injrho2_c/injrho1_c,injrho3_c/injrho2_c,'rx', label='injections')
altVeto_c = (
    (rho1_c == rho1H_c) | (rho1_c == rho1L_c) |
    (rho2_c == rho2H_c) | (rho2_c == rho2L_c) |
    (rho3_c == rho3H_c) | (rho3_c == rho3L_c)
)
ax.plot(rho2_c[altVeto_c]/rho1_c[altVeto_c],
        rho3_c[altVeto_c]/rho2_c[altVeto_c],
        'ms',mec='C0',mfc='None',label='single detector')

# veto13_c = vetoNum_c == 13
# ax.plot(rho2_c[veto13_c]/rho1_c[veto13_c],
#         rho3_c[veto13_c]/rho2_c[veto13_c],
#         'mo',mec='C1',mfc='None',label='line vetoed')

injAltVeto_c = (
    (injrho1_c == injrho1H_c) | (injrho1_c == injrho1L_c) |
    (injrho2_c == injrho2H_c) | (injrho2_c == injrho2L_c) |
    (injrho3_c == injrho3H_c) | (injrho3_c == injrho3L_c)
)
ax.plot(injrho2_c[injAltVeto_c]/injrho1_c[injAltVeto_c],
        injrho3_c[injAltVeto_c]/injrho2_c[injAltVeto_c],
        'ms',mec='C0',mfc='None')

ax.grid(True)
ax.set_xlabel(r'$\rho_{\rm level\ 2}/\rho_{\rm level\ 1}$')
ax.set_ylabel(r'$\rho_{\rm level\ 3}/\rho_{\rm level\ 2}$')
ax.set_xlim([xmin,xmax])
ax.set_ylim([ymin,ymax])
ax.legend(numpoints=1)

fig.savefig('followupRatio.eps',bbox_inches='tight')
fig.savefig('followupRatio.svg',bbox_inches='tight')
plt.close(fig)

Nmonte = 10**6
np.random.seed(20220215)
Porb_I = stats.norm(loc=WangPorbSec,scale=WangdPorbSec).rvs(size=Nmonte)
Tasc_I = stats.norm(loc=WangTascGPS,scale=WangdTascSec).rvs(size=Nmonte)

O3inFullPorb_I = (
    ( Porb_I > (WangPorbSec-3*WangdPorbSec) )
    & ( Porb_I < (WangPorbSec+3*WangdPorbSec) )
)
print('Monte Carlo finds %g of Porb prior in +/- 3 sigma' % np.mean(O3inFullPorb_I))
print('Check: should be %g' % (stats.norm.cdf(3)-stats.norm.cdf(-3)))

O3Tasc_I = Tasc_I + norb * Porb_I
PorbShearCorr_I = Porb_I - dPorbdTascShearCorr * (O3Tasc_I-TascGPS)
PorbShearUsed_I = Porb_I - dPorbdTascShearUsed * (O3Tasc_I-TascGPS)

O3inTP3sig_I = (
    ((Tasc_I-WangTascGPS)/WangdTascSec)**2 + ((Porb_I-WangPorbSec)/WangdPorbSec)**2
    < 3**2
)

O3inTP3p3sig_I = (
    ((Tasc_I-WangTascGPS)/WangdTascSec)**2 + ((Porb_I-WangPorbSec)/WangdPorbSec)**2
    < (3.3)**2
)

O3inTP3sigCorr_I = (
    ((O3Tasc_I-TascGPS)/dTascSec)**2
    + ((PorbShearCorr_I-WangPorbSec)/dPtildeSec)**2
    < 3**2
)

O3inTP3p3sigCorr_I = (
    ((O3Tasc_I-TascGPS)/dTascSec)**2
    + ((PorbShearCorr_I-WangPorbSec)/dPtildeSec)**2
    < (3.3)**2
)

O3inTP3sigUsed_I = (
    ((O3Tasc_I-TascGPS)/dTascSec)**2
    + ((PorbShearUsed_I-WangPorbSec)/dPtildeSec)**2
    < 3**2
)

O3inTP3p3sigUsed_I = (
    ((O3Tasc_I-TascGPS)/dTascSec)**2
    + ((PorbShearUsed_I-WangPorbSec)/dPtildeSec)**2
    < (3.3)**2
)

Tsig_I = (O3Tasc_I - Torigin)/dTascSec

innerFracCorr = np.mean(O3inTP3p3sigCorr_I & (np.abs(Tsig_I) < 1))
innerFracUsed = np.mean(O3inTP3p3sigUsed_I & (np.abs(Tsig_I) < 1))

outerFracCorr = np.mean(O3inTP3p3sigCorr_I & (np.abs(Tsig_I) < 3))
outerFracUsed = np.mean(O3inTP3p3sigUsed_I & (np.abs(Tsig_I) < 3))

tabfile = open('O3ScoX1CrossCorr_dat.tex','w')
tabfile.write(r'\newcommand{\LSCRaHr}{%d}' % ScoX1raHMS[0])
tabfile.write('\n')
tabfile.write(r'\newcommand{\LSCRaMin}{%d}' % ScoX1raHMS[1])
tabfile.write('\n')
tabfile.write(r'\newcommand{\LSCRaSec}{%.4f}' % ScoX1raHMS[2])
tabfile.write('\n')
tabfile.write(r'\newcommand{\LSCNegDecDeg}{%d}' % ScoX1negdecDMS[0])
tabfile.write('\n')
tabfile.write(r'\newcommand{\LSCNegDecMin}{%d}' % ScoX1negdecDMS[1])
tabfile.write('\n')
tabfile.write(r'\newcommand{\LSCNegDecSec}{%.1f}' % ScoX1negdecDMS[2])
tabfile.write('\n')
tabfile.write(r'\newcommand{\WangPorbSec}{%.8s}'% WangPorbSec)
tabfile.write('\n')
tabfile.write(r'\newcommand{\WangdPorbSec}{%.5s}'% WangdPorbSec)
tabfile.write('\n')
tabfile.write(r'\newcommand{\WangTascGPS}{%.0f}' % WangTascGPS)
tabfile.write('\n')
tabfile.write(r'\newcommand{\WangTascDayTime}{%s}'%
              gpstime.gps_to_str(int(round(WangTascGPS)),daytimefmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\WangTascDay}{%s}'%
              gpstime.gps_to_str(int(round(WangTascGPS)),dayfmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\WangdTascGPS}{%.0f}'% WangdTascSec)
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiTascGPS}{%.0f}' % TascGPS)
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiTascDayTime}{%s}'%
              gpstime.gps_to_str(int(round(TascGPS)),daytimefmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiTascDay}{%s}'%
              gpstime.gps_to_str(int(round(TascGPS)),dayfmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiidTascGPS}{%.0f}'% dTascSec)
tabfile.write('\n')
tabfile.write(r'\newcommand{\dPtildeSec}{%.5s}'% dPtildeSec)
tabfile.write('\n')
tabfile.write(r'\newcommand{\dPorbdTascShearCorr}{%s}'%
              latexscinot('%.2f',dPorbdTascShearCorr))
tabfile.write('\n')
tabfile.write(r'\newcommand{\dPorbdTascShearUsed}{%s}'%
              latexscinot('%.2f',dPorbdTascShearUsed))
tabfile.write('\n')
tabfile.write(r'\newcommand{\WangAsiniMin}{%.2f}'% apMin)
tabfile.write('\n')
tabfile.write(r'\newcommand{\WangAsiniMax}{%.2f}'% apMax)
tabfile.write('\n')
tabfile.write(r'\newcommand{\htorque}{%s}'
              % latexscinot( '%.1f', h0600Hz) )
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiaStartGPS}{%.0f}' % O3aStartGPS)
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiaStartDayTime}{%s}'%
              gpstime.gps_to_str(int(round(O3aStartGPS)),daytimefmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiaStartDay}{%s}'%
              gpstime.gps_to_str(int(round(O3aStartGPS)),dayfmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiaEndGPS}{%.0f}' % O3aEndGPS)
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiaEndDayTime}{%s}'%
              gpstime.gps_to_str(int(round(O3aEndGPS)),daytimefmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiaEndDay}{%s}'%
              gpstime.gps_to_str(int(round(O3aEndGPS)),dayfmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiibStartGPS}{%.0f}' % O3bStartGPS)
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiibStartDayTime}{%s}'%
              gpstime.gps_to_str(int(round(O3bStartGPS)),daytimefmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiibStartDay}{%s}'%
              gpstime.gps_to_str(int(round(O3bStartGPS)),dayfmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiibEndGPS}{%.0f}' % O3bEndGPS)
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiibEndDayTime}{%s}'%
              gpstime.gps_to_str(int(round(O3bEndGPS)),daytimefmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiibEndDay}{%s}'%
              gpstime.gps_to_str(int(round(O3bEndGPS)),dayfmt))
tabfile.write('\n')
tabfile.write(r'\newcommand{\onesigpct}{%.1f}'
              % (100*stats.chi2.cdf(1.**2,df=2)))
tabfile.write('\n')
tabfile.write(r'\newcommand{\twosigpct}{%.1f}'
              % (100*stats.chi2.cdf(2.**2,df=2)))
tabfile.write('\n')
tabfile.write(r'\newcommand{\threesigpct}{%.1f}'
              % (100*stats.chi2.cdf(3.**2,df=2)))
tabfile.write('\n')
tabfile.write(r'\newcommand{\innerpctCorr}{%.1f}'
              % (100*(innerFracCorr)))
tabfile.write('\n')
tabfile.write(r'\newcommand{\outerpctCorr}{%.1f}'
              % (100*(outerFracCorr)))
tabfile.write('\n')
tabfile.write(r'\newcommand{\innerpctUsed}{%.1f}'
              % (100*(innerFracUsed)))
tabfile.write('\n')
tabfile.write(r'\newcommand{\outerpctUsed}{%.1f}'
              % (100*(outerFracUsed)))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiNorb}{%d}' % norb)
tabfile.write('\n')
tabfile.write(r'\newcommand{\mismatchMax}{%g}'
              % (0.25))
tabfile.write('\n')
tabfile.write(r'\newcommand{\mismatchMaxP}{%g}'
              % (0.25/4))
tabfile.write('\n')
tabfile.write(r'\newcommand{\nCellsFine}{%g}'
              % (nCells_F[0]))
tabfile.write('\n')
tabfile.write(r'\newcommand{\nCellsCoarse}{%g}'
              % (nCells_F[-1]))
tabfile.write('\n')
tabfile.write(r'\newcommand{\numUnvetoed}{%d}' % np.sum(vetoNum_c==0))
tabfile.write('\n')
tabfile.write(r'\newcommand{\numAltvetoed}{%d}'
              % np.sum((vetoNum_c==0) & altVeto_c))
tabfile.write('\n')
tabfile.write(r'\newcommand{\numInjFollowup}{%d}' % len(injvetoNum_c))
tabfile.write('\n')
tabfile.write(r'\newcommand{\numInjUnvetoed}{%d}'
              % np.sum(injvetoNum_c==0))
tabfile.write('\n')
tabfile.write(r'\newcommand{\numInjAltvetoed}{%d}'
              % np.sum((injvetoNum_c==0) & injAltVeto_c))
tabfile.write('\n')
tabfile.write(r'\newcommand{\numInjVetoedi}{%d}'
              % np.sum((injvetoNum_c==1)|(injvetoNum_c==11)))
tabfile.write('\n')

tabfile.write(r'\newcommand{\numInjVetoedii}{%d}'
              % np.sum((injvetoNum_c==2)|(injvetoNum_c==12)))
tabfile.write('\n')
tabfile.write(r'\newcommand{\numInjVetoediii}{%d}'
              % np.sum((injvetoNum_c==3)|(injvetoNum_c==13)))
tabfile.write('\n')
cp = ConfigParser()
cp.optionxform = str
cp.read('injection_efficiency_0025_0500.ini')
tabfile.write(r'\newcommand{\numInjTotal}{%s}'
              % cp.get('h0eff','numinj'))
tabfile.write('\n')
tabfile.write(r'\newcommand{\numInjTotalFound}{%s}'
              % cp.get('h0eff','numinjFound'))
tabfile.write('\n')
tabfile.write(r'\newcommand{\numInjho}{%s}'
              % cp.get('h0','numinj'))
tabfile.write('\n')
tabfile.write(r'\newcommand{\numInjhoFound}{%s}'
              % cp.get('h0','numinjFound'))
tabfile.write('\n')
tabfile.write(r'\newcommand{\Oiiihoefffudge}{%s}'
              % cp.get('h0eff','fudge'))
tabfile.write('\n')
tabfile.write(r'\newcommand{\Oiiihofudge}{%s}'
              % cp.get('h0','fudge'))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiTrun}{%s}'
              % latexscinot( '%.2f', Trun))
tabfile.write('\n')
tabfile.write(r'\newcommand{\OiiiSWLoss}{%.3f}' % SWLoss )
tabfile.write('\n')
tabfile.write(r'\newcommand{\TmaxMax}{%.0f}' % TmaxMax )
tabfile.write('\n')
tabfile.close()

tabfile = open('SearchSummary.tex','w')
tabfile.write(r'\tablehead{')
tabfile.write('\n')
tabfile.write(r'\multicolumn{2}{c}{$f_0$(Hz)} &')
tabfile.write('\n')
tabfile.write(r'\colhead{$\Tsft$} &')
tabfile.write('\n')
tabfile.write(r'\multicolumn{2}{c}{$\Tmax$(s)} &')
tabfile.write('\n')
tabfile.write(r'\colhead{$\rho$} &')
tabfile.write('\n')
tabfile.write('\colhead{number of} &')
tabfile.write('\n')
tabfile.write(r'\colhead{expected Gauss} &')
tabfile.write('\n')
tabfile.write(r'\multicolumn{4}{c}{followup level} \\')
tabfile.write('\n')

tabfile.write(r'\colhead{min} &')
tabfile.write('\n')
tabfile.write(r'\colhead{max} &')
tabfile.write('\n')
tabfile.write(r'\colhead{(s)} &')
tabfile.write('\n')
tabfile.write(r'\colhead{min} &')
tabfile.write('\n')
tabfile.write(r'\colhead{max} &')
tabfile.write('\n')
tabfile.write(r'\colhead{thresh\tablenotemark{a}} &')
tabfile.write('\n')
tabfile.write(r'\colhead{templates} &')
tabfile.write('\n')
tabfile.write(r'\colhead{false alarms\tablenotemark{b}} &')
tabfile.write('\n')
tabfile.write(r'\colhead{0\tablenotemark{c}} &')
tabfile.write('\n')
tabfile.write(r'\colhead{1\tablenotemark{d}} &')
tabfile.write('\n')
tabfile.write(r'\colhead{2\tablenotemark{e}} &')
tabfile.write('\n')
tabfile.write(r'\colhead{3\tablenotemark{f}}')
tabfile.write('\n')
tabfile.write(r'}')
tabfile.write('\n')

boundaries_S = np.array([25,50,100,150,200,300,400,
                         600,800,1200,1600])
fLo_S = boundaries_S[:-1]
fHi_S = boundaries_S[1:]

nSumms = len(fLo_S)

tabfile.write(r'\startdata')
tabfile.write('\n')
numTemplatesInSumm = np.empty(nSumms,dtype=int)
for s in range(nSumms):
    if (fLo_S[s] < 400):
        rhoth = 6.3
    elif (fLo_S[s] < 600):
        rhoth = 6.2
    else:
        rhoth = 5.8

    bandMask_F = ( (fLo_F >=fLo_S[s]) & (fHi_F <=fHi_S[s]) )
    tabfile.write(r'%d &' % fLo_S[s])
    tabfile.write('\n')
    tabfile.write(r'%d &' % fHi_S[s])
    tabfile.write('\n')
    # The Tsft values should all be the same for a band
    tabfile.write(r'%d &' % min(Tsft_F[bandMask_F]))
    tabfile.write('\n')
    tabfile.write(r'%d &' % min(TmaxLo_F[bandMask_F]))
    tabfile.write('\n')
    tabfile.write(r'%d &' % max(TmaxHi_F[bandMask_F]))
    tabfile.write('\n')
    tabfile.write(r'%.1f &' % rhoth)
    tabfile.write('\n')
    tmpltMask_u = ( (fmid_u >= fLo_S[s]) & (fmid_u <= fHi_S[s]) )
    numTmplt = np.sum(nTmplt_u[tmpltMask_u])
    tabfile.write(r'$%s$ &' % latexscinot(r'%.2f',numTmplt))
    tabfile.write('\n')
    tabfile.write(r'%.1f &' % (numTmplt * stats.norm.sf(rhoth)))
    tabfile.write('\n')
    candMask_c = ( (freq_c >= fLo_S[s]) & (freq_c <= fHi_S[s]) )
    vetoNumBand_c = vetoNum_c[candMask_c]
    tabfile.write(r'%d &' % len(vetoNumBand_c))
    tabfile.write('\n')
    tabfile.write(r'%d &' % np.sum(vetoNumBand_c != 11))
    tabfile.write('\n')
    tabfile.write(r'%d &' % np.sum((vetoNumBand_c == 0)
                                   | (vetoNumBand_c == 13)
                                   | (vetoNumBand_c == 3))
                  )
    tabfile.write('\n')
    tabfile.write(r'%d' % np.sum(vetoNumBand_c == 0))
    tabfile.write('\n')
    tabfile.write(r'\\')
    tabfile.write('\n')
tabfile.write(r'\enddata')
tabfile.write('\n')
tabfile.close()
