#!/usr/bin/env python3
# script to produce figures related to Sco X-1 upper limits

# Copyright (C) 2022 John T. Whelan

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with with program; see the file COPYING. If not, see
# <https://www.gnu.org/licenses/>.

import numpy as np
import matplotlib
matplotlib.use("Agg")
from matplotlib import rc
matplotlib.rc('text', usetex=True)
matplotlib.rc('font', family='serif')
import matplotlib.pyplot as plt
from matplotlib import cm
import matplotlib.font_manager as ftmgr
import numpy as np
from configparser import ConfigParser
from scipy import stats
import h5py
import tbrange
from scipy.interpolate import interp1d

# Cribbed from https://stackoverflow.com/questions/22263807/how-is-order-of-items-in-matplotlib-legend-determined
#  Returns tuple of handles, labels for axis ax, after reordering them to conform to the label order `order`, and if unique is True, after removing entries with duplicate labels.

def reorderLegend(ax=None,order=None,unique=False,loc="lower right",ncol=1,
                  handlelength=2.6):
    if ax is None: ax=plt.gca()
    handles, labels = ax.get_legend_handles_labels()
    labels, handles = zip(*sorted(zip(labels, handles), key=lambda t: t[0])) # sort both labels and handles by labels
    if order is not None: # Sort according to a given list (not necessarily complete)
        keys=dict(zip(order,range(len(order))))
        labels, handles = zip(*sorted(zip(labels, handles), key=lambda t,keys=keys: keys.get(t[0],np.inf)))
    if unique:  labels, handles= zip(*unique_everseen(zip(labels,handles), key = labels)) # Keep only the first of each handle
    #     ax.legend(handles, labels)
    # Hack to propagate the properties I used
    ax.legend(handles, labels, loc=loc,
              handlelength=handlelength, numpoints=6,
              prop=ftmgr.FontProperties(size=8), framealpha=1.0, ncol=ncol)
    return(handles, labels)

def unique_everseen(seq, key=None):
    seen = set()
    seen_add = seen.add
    return [x for x,k in zip(seq,key) if not (k in seen or seen_add(k))]

## This conversion uses the convention that h0eff^2 <= h0^2
## (i.e., they are equal for circular polarization)
def h0effsq_by_h0sq(cosisq):
    return 0.5 * ( ( 0.5*(1.+cosisq) )**2 + cosisq )

# Torque balance numbers
BildstenHc600Hz = 4e-27 * np.sqrt(39)
BradyH0600Hz = np.sqrt(15./32.) * BildstenHc600Hz
h0600Hz = 2. * BradyH0600Hz

fmin=25.
fmax=1600.
f_f = np.logspace(np.log10(fmin),np.log10(fmax),200)
h0tb_f = h0600Hz*np.sqrt(600./f_f)
h0effTBcirc_f = h0tb_f * np.sqrt(h0effsq_by_h0sq(1))
cosisq44 = ( np.cos(np.radians(44.)) )**2
cosisq44gauss95 = (
    np.cos(np.radians(stats.norm(loc=44,scale=6).ppf(0.95)))
)**2
cosisq44gauss05 = (
    np.cos(np.radians(stats.norm(loc=44,scale=6).ppf(0.05)))
)**2
h0effTB44_f = h0tb_f * np.sqrt(h0effsq_by_h0sq(cosisq44))
h0effTB44gauss95_f = h0tb_f * np.sqrt(h0effsq_by_h0sq(cosisq44gauss95))
h0effTB44fgauss05_f = h0tb_f * np.sqrt(h0effsq_by_h0sq(cosisq44gauss05))
h0effTBlin_f = h0tb_f * np.sqrt(h0effsq_by_h0sq(0))

print('Factors to multiply h0eff by to get h0:')
print('Linear polarization (iota=90 deg, cosi=0): %g' %
      (1./np.sqrt((h0effsq_by_h0sq(0.**2))))
)
print('Circular polarization (iota=0 deg, cosi=1): %g' %
      (1./np.sqrt((h0effsq_by_h0sq(1.**2))))
)
print('95%% isotropic marginalization (iota=%g deg, cosi=0.05): %g' %
      (
          np.degrees(np.arccos(0.05)),
          1./np.sqrt((h0effsq_by_h0sq(0.05**2)))
      )
)
print('Assume most likely inclination (iota=44 deg, cosi=%g): %g' %
      (
          np.sqrt(cosisq44),
          1./np.sqrt((h0effsq_by_h0sq(cosisq44)))
      )
)
print('95%% marginalization with Gaussian iota (iota=%g deg, cosi=%g): %g' %
      (
          stats.norm(loc=44,scale=6).ppf(0.95),
          np.sqrt(cosisq44gauss95),
          1./np.sqrt((h0effsq_by_h0sq(cosisq44gauss95)))
      )
)

(
    O1CCf_f,O1CCh0UL_f
) = np.loadtxt('O1ScoX1CrossCorrUL.dat',unpack=True,
               usecols=(0,2))

# Datafile from https://www.aei.mpg.de/continuouswaves/CrossCorr-O2-20-180
(
    O2CCf_f, O2CCh0UL_f
) = np.loadtxt("upperLimit_0.95.txt", unpack=True,
               usecols=(0,2))

(
    O3CCf_f,O3CCh0effUL_f,O3CCh0UL_f
) = np.loadtxt('O3ScoX1CrossCorrUL.dat',unpack=True,
               usecols=(0,1,2))

(
    O3VIf_f, O3VIh0effUL_f
) = np.loadtxt("O3_ScoX1_HMM_UL.txt", unpack=True,
               usecols=(0,1), comments='%', delimiter=',' )

# Datafile from https://dcc.ligo.org/LIGO-G2002165/public
O3RMUL = h5py.File("NBR_scox1_ul.hdf5","r")
O3RMf_f = np.array(O3RMUL['f_all'])
O3RMh0UL_f = np.array(O3RMUL['ul_scox1'])
O3RMh0effUL_f = O3RMh0UL_f * np.sqrt((h0effsq_by_h0sq(0.05**2)))

fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)

ax.loglog(O1CCf_f,O1CCh0UL_f,'c-',color='C0',label='CrossCorr O1')
ax.loglog(O2CCf_f,O2CCh0UL_f,'g-',color=(0.5,0.2,0.0),label='CrossCorr O2')
ax.loglog(O3CCf_f,O3CCh0UL_f,'k-',label='CrossCorr O3')
ax.loglog(f_f,h0tb_f,'b--',label=r'torque balance')

ax.legend(loc="lower right",handlelength=2.6, numpoints=6,
          prop=ftmgr.FontProperties(size=8))
reorderLegend(ax,['CrossCorr O1','CrossCorr O2','CrossCorr O3',
                  'torque balance'])

ax.set_xlabel(r'GW Frequency (Hz)')
ax.set_ylabel(r'$h_0$ upper limit')
ax.set_title(r'Marginalized 95\% Upper Limits')
ax.grid(True)
tickvals = [int(fmin),50,100,200,500,1000,int(fmax)]
ticklabs = ['%d' % x for x in tickvals]
ax.set_xticks(tickvals)
ax.set_xticklabels(ticklabs)
tickvals = [2e-26,5e-26,1e-25,2e-25,5e-25,1e-24]
ticklabs = [r'$2\times10^{-26}$',r'$5\times10^{-26}$',
            r'$10^{-25}$',r'$2\times10^{-25}$',r'$5\times10^{-25}$',
            r'$10^{-24}$']
ax.set_yticks(tickvals)
ax.set_yticklabels(ticklabs)
ax.set_xlim(fmin,fmax)
ax.set_ylim(2e-26,1e-24)

fig.savefig('upperlimits_o1o2o3_h0_big.pdf',bbox_inches='tight')
plt.close(fig)


fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)

ax.fill_between(O3CCf_f,O3CCh0effUL_f,
                O3CCh0effUL_f/np.sqrt((h0effsq_by_h0sq(0))),
                color=[0.7,0.7,0.7],alpha=0.8,label='UL for diff pols')
tbrange9min,tbrange9max = tbrange.tbrange(f_f)
tbrange8min,tbrange8max = tbrange.tbrange8(f_f)
ax.fill_between(f_f,tbrange9min,tbrange9max,
                linestyle='--',edgecolor='k',facecolor='wheat',alpha=0.8,
                label='GS9')
ax.fill_between(f_f,tbrange8min,tbrange8max,
                linestyle='--',edgecolor='darkred',
                facecolor='lightcoral',alpha=0.3,
                label='GS8')
ax.loglog(f_f,tbrange.tbra(f_f,1),'r-.',label=r'na\"{\i}ve ($r=r_A$)')
ax.loglog(f_f,tbrange.tbnaive(f_f),'b--',label=r'na\"{\i}ve ($r=R_*$)')

ax.legend(loc="upper right",handlelength=3.0, numpoints=6,
          prop=ftmgr.FontProperties(size=8), ncol=2, framealpha=1.0)

ax.set_xlabel(r'GW Frequency (Hz)')
ax.set_ylabel(r'$h_0$')
ax.set_title(r'Torque balance models')
ax.grid(True)
tickvals = [int(fmin),50,100,200,500,1000,int(fmax)]
ticklabs = ['%d' % x for x in tickvals]
ax.set_xticks(tickvals)
ax.set_xticklabels(ticklabs)
tickvals = [2e-26,5e-26,1e-25,2e-25,5e-25,1e-24]
ticklabs = [r'$2\times10^{-26}$',r'$5\times10^{-26}$',
            r'$10^{-25}$',r'$2\times10^{-25}$',r'$5\times10^{-25}$',
            r'$10^{-24}$']
ax.set_yticks(tickvals)
ax.set_yticklabels(ticklabs)
ax.set_xlim(fmin,fmax)
ax.set_ylim(2e-26,1e-24)

fig.savefig('torquebalance_h0_big.pdf',bbox_inches='tight')
plt.close(fig)

datfile = open('O3CrossCorrUL.dat','w')
datfile.write(r'# Final O3 CrossCorr upper limits including adjustment from injections')
datfile.write('\n')
datfile.write(r'# freq(Hz) h0circ95 h0lin95')
datfile.write('\n')
for f in range(len(O3CCf_f)):
    datfile.write(r'%.4f %.4g %.4g'
                  % (O3CCf_f[f],
                     O3CCh0effUL_f[f],
                     O3CCh0effUL_f[f]/np.sqrt((h0effsq_by_h0sq(0))))
                  )
    datfile.write('\n')

Mmaxplot = 2.5
(GPPV_mlimcirc,Mmax) = tbrange.masslimit(O3CCf_f,O3CCh0effUL_f,"GPPV.mr")
(GPPV_mlim44,Mmax) = tbrange.masslimit(O3CCf_f,
                                  O3CCh0effUL_f/np.sqrt((h0effsq_by_h0sq(cosisq44))),
                                   "GPPV.mr")
(GPPV_mlimlin,Mmax) = tbrange.masslimit(O3CCf_f,
                                   O3CCh0effUL_f/np.sqrt((h0effsq_by_h0sq(0))),
                                   "GPPV.mr")

fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)

ax.set_xscale('log')
ax.fill_between(O3CCf_f,GPPV_mlimcirc,0.9,color=(0.5,0.5,1.0),
                label='excluded for $\iota=0^\circ$')
ax.fill_between(O3CCf_f,GPPV_mlim44,0.9,color=(0.5,1.0,0.5),
                label='excluded for $\iota\le 44^\circ$')
ax.fill_between(O3CCf_f,GPPV_mlimlin,0.9,color=(1.0,0.5,0.5),
                label='excluded for all $\iota$')
ax.fill_between(O3CCf_f,Mmax,Mmaxplot,color='grey')
ax.axhline(y=1.4,color='C2')
ax.set_xlabel(r'GW Frequency (Hz)')
ax.set_ylabel(r'NS Mass ($M_\odot$)')
ax.set_title(r'Torque Balance Constraints w/GPPVA EOS')
ax.grid(True)
tickvals = [int(fmin),50,100,200,500,1000,int(fmax)]
ticklabs = ['%d' % x for x in tickvals]
ax.set_xticks(tickvals)
ax.set_xticklabels(ticklabs)
ax.set_xlim(fmin,fmax)
ax.set_ylim(1.0,Mmaxplot)
ax.legend(loc='center right',framealpha=1.0)

fig.savefig('GPPVlimits_big.pdf',bbox_inches='tight')
plt.close(fig)

Mmaxplot = 2.5
(gr15_mlimcirc,Mmax) = tbrange.masslimit(O3CCf_f,O3CCh0effUL_f,"gr15eos.mr")
(gr15_mlim44,Mmax) = tbrange.masslimit(O3CCf_f,
                                  O3CCh0effUL_f/np.sqrt((h0effsq_by_h0sq(cosisq44))),
                                   "gr15eos.mr")
(gr15_mlimlin,Mmax) = tbrange.masslimit(O3CCf_f,
                                   O3CCh0effUL_f/np.sqrt((h0effsq_by_h0sq(0))),
                                   "gr15eos.mr")

fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)

ax.set_xscale('log')
ax.fill_between(O3CCf_f,gr15_mlimcirc,0.9,color=(0.5,0.5,1.0),
                label='excluded for $\iota=0^\circ$')
ax.fill_between(O3CCf_f,gr15_mlim44,0.9,color=(0.5,1.0,0.5),
                label='excluded for $\iota\le 44^\circ$')
ax.fill_between(O3CCf_f,gr15_mlimlin,0.9,color=(1.0,0.5,0.5),
                label='excluded for all $\iota$')
ax.fill_between(O3CCf_f,Mmax,Mmaxplot,color='grey')
ax.axhline(y=1.4,color='C2')
ax.set_xlabel(r'GW Frequency (Hz)')
ax.set_ylabel(r'NS Mass ($M_\odot$)')
ax.set_title(r'Torque Balance Constraints w/GR15 EOS')
ax.grid(True)
tickvals = [int(fmin),50,100,200,500,1000,int(fmax)]
ticklabs = ['%d' % x for x in tickvals]
ax.set_xticks(tickvals)
ax.set_xticklabels(ticklabs)
# ax.set_yticks(tickvals)
# ax.set_yticklabels(ticklabs)
ax.set_xlim(fmin,fmax)
ax.set_ylim(1.0,Mmaxplot)
ax.legend(loc='center right',framealpha=1.0)

fig.savefig('gr15limits_big.pdf',bbox_inches='tight')
plt.close(fig)

GPPV_cosilim = tbrange.cosilimit(O3CCf_f,O3CCh0effUL_f,"GPPV.mr")
GPPV_cosilim = np.minimum(GPPV_cosilim,1)

fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)

ax.set_xscale('log')
ax.fill_between(O3CCf_f,np.degrees(np.arccos(GPPV_cosilim)),-1,
                color=(1.0,0.5,0.5),
                label='excluded for $M=1.4M_\odot$')
ax.axhline(y=44,color='C2')
ax.set_xlabel(r'GW Frequency (Hz)')
ax.set_ylabel(r'NS Spin Inclination $\iota$')
ax.set_title(r'Torque Balance Constraints w/GPPVA EOS')
ax.grid(True)
tickvals = [int(fmin),50,100,200,500,1000,int(fmax)]
ticklabs = ['%d' % x for x in tickvals]
ax.set_xticks(tickvals)
ax.set_xticklabels(ticklabs)
tickvals = np.arange(0,91,15)
ticklabs = [r'$%d^\circ$' % x for x in tickvals]
ax.set_yticks(tickvals)
ax.set_yticklabels(ticklabs)
ax.set_xlim(fmin,fmax)
ax.set_ylim(0,90)
ax.legend(loc='upper right',framealpha=1.0)

fig.savefig('GPPValpha_big.pdf',bbox_inches='tight')
plt.close(fig)

gr15_cosilim = tbrange.cosilimit(O3CCf_f,O3CCh0effUL_f,"gr15eos.mr")
gr15_cosilim = np.minimum(gr15_cosilim,1)

fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)

ax.set_xscale('log')
ax.fill_between(O3CCf_f,np.degrees(np.arccos(gr15_cosilim)),-1,
                color=(1.0,0.5,0.5),
                label='excluded for $M=1.4M_\odot$')
ax.axhline(y=44,color='C2')
ax.set_xlabel(r'GW Frequency (Hz)')
ax.set_ylabel(r'NS Spin Inclination $\iota$')
ax.set_title(r'Torque Balance Constraints w/GR15 EOS')
ax.grid(True)
tickvals = [int(fmin),50,100,200,500,1000,int(fmax)]
ticklabs = ['%d' % x for x in tickvals]
ax.set_xticks(tickvals)
ax.set_xticklabels(ticklabs)
tickvals = np.arange(0,91,15)
ticklabs = [r'$%d^\circ$' % x for x in tickvals]
ax.set_yticks(tickvals)
ax.set_yticklabels(ticklabs)
ax.set_xlim(fmin,fmax)
ax.set_ylim(0,90)
ax.legend(loc='upper right',framealpha=1.0)

fig.savefig('gr15alpha_big.pdf',bbox_inches='tight')
plt.close(fig)

fig = plt.figure();
fig.set_size_inches(5,3);

ax = fig.add_subplot(111)

ax.loglog(f_f,h0effTB44_f,'b-.',label=r'tb w/$\iota=44^{\circ}$')
ax.fill_between(f_f,h0effTBlin_f,h0effTBcirc_f,linewidth=0,
                facecolor=[0.7,0.7,1.0])
ax.fill_between(f_f,h0effTB44gauss95_f,h0effTB44fgauss05_f,linewidth=0,
                facecolor=[0.5,0.6,1.0])
ax.loglog(O3RMf_f,O3RMh0effUL_f,'m-',
          color=(1.0,0.5,1.0),
          alpha=0.4,label='Radiometer O3')
ax.loglog(O3VIf_f,O3VIh0effUL_f,'md',ms=1,color=(0.0,0.3,0.0),
          label='Viterbi O3')
ax.loglog(O3CCf_f,O3CCh0effUL_f,'k-',label='CrossCorr O3')
ax.legend(loc='upper right',handlelength=3.0, numpoints=6,
          prop=ftmgr.FontProperties(size=10))
reorderLegend(ax,['Radiometer O3','Viterbi O3','CrossCorr O3',
                  r'tb w/$\iota=44^{\circ}$'],loc='upper center',
              handlelength=3.5)

ax.set_xlabel(r'GW Frequency (Hz)')
ax.set_ylabel(r'$h_0^{\rm eff}$ upper limit')
ax.set_title(r'Unmarginalized 95\% Upper Limits')
ax.grid(True)
tickvals = [int(fmin),50,100,200,500,1000,int(fmax)]
ticklabs = ['%d' % x for x in tickvals]
ax.set_xticks(tickvals)
ax.set_xticklabels(ticklabs)
tickvals = [2e-26,5e-26,1e-25,2e-25,5e-25,1e-24]
ticklabs = [r'$2\times10^{-26}$',r'$5\times10^{-26}$',
            r'$10^{-25}$',r'$2\times10^{-25}$',r'$5\times10^{-25}$',
            r'$10^{-24}$']
ax.set_yticks(tickvals)
ax.set_yticklabels(ticklabs)
ax.set_xlim(fmin,fmax)
ax.set_ylim(2e-26,1e-24)

fig.savefig('upperlimits_o1o2o3_h0eff_big.pdf',bbox_inches='tight')
plt.close(fig)
